package com.safeway.epe.avrokafkawithoutschemaregistry.configuration;

import com.safeway.epe.avrokafkawithoutschemaregistry.serializer.KafkaAvroDeserialize;
import com.safeway.epe.model.Promotion;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.Deserializer;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.springframework.stereotype.Component;

import java.util.Properties;

@Component
public interface PropertiesConfiguration {
    public static <T> KafkaProducer<String, T> getProperties(KafkaProperties kafkaProperties) {
        final Properties properties = new Properties();
        properties.setProperty(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, kafkaProperties.getBootStrapServers());
        properties.setProperty(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, kafkaProperties.getKeySerializer());
        properties.setProperty(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, kafkaProperties.getValueSerializer());
        properties.setProperty(ProducerConfig.ACKS_CONFIG, kafkaProperties.getAcks());
        properties.setProperty(ProducerConfig.RETRIES_CONFIG, kafkaProperties.getRetries());
        return new KafkaProducer<String, T>(properties);
    }
    public static <T> KafkaConsumer<String, T> consumerProperties(KafkaProperties kafkaConsumerProperties) {
        final Properties properties = new Properties();
        properties.setProperty(ConsumerConfig.GROUP_ID_CONFIG, kafkaConsumerProperties.getGroupId());
        properties.setProperty(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, kafkaConsumerProperties.getBootStrapServers());
        properties.setProperty(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, kafkaConsumerProperties.getKeyDeSerializer());
        properties.setProperty(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, kafkaConsumerProperties.getValueDeSerializer());
        return new KafkaConsumer<String, T>(properties, new StringDeserializer(), (Deserializer<T>) new KafkaAvroDeserialize<>(Promotion.class));
    }
}
