package com.safeway.epe.avrokafkawithoutschemaregistry.consumer;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.safeway.epe.avrokafkawithoutschemaregistry.configuration.KafkaProperties;
import com.safeway.epe.avrokafkawithoutschemaregistry.serializer.KafkaAvroSerialize;
import com.safeway.epe.model.Promotion;
import org.apache.kafka.clients.consumer.*;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.TopicPartition;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.springframework.test.context.junit4.SpringRunner;

import java.time.Duration;
import java.util.*;

@RunWith(SpringRunner.class)
public class PromotionConsumerTest {
    public static String topic = "promotion_spark2";
    @InjectMocks
    private PromotionConsumer promotionConsumer;
    @Mock
    private KafkaProperties kafkaProperties;
    @Mock
    private KafkaConsumer<String, Promotion> consumer;

    @Test
    public void testConsumeMesage() throws InterruptedException, JsonProcessingException {
        ConsumerRecords<String, Promotion> records = null;
        ConsumerRecords<String, Promotion> consumerRecords = buildConsumerRecords();
        List<ConsumerRecord<String, Promotion>> consumerRecord = consumerRecords.records(new TopicPartition(topic, 0));
        Mockito.doNothing().when(consumer).subscribe(Collections.singleton(topic));
        Mockito.when(kafkaProperties.getTopic()).thenReturn(topic);
        Mockito.when(consumer.poll(Duration.ofMillis(100))).thenReturn(consumerRecords);
        ConsumerRecords<String, Promotion> record = promotionConsumer.consumeMesage(consumer);
        Thread.sleep(1000);
        consumer.close(Duration.ofMillis(1000));
    }

    private static ConsumerRecords<String, Promotion> buildConsumerRecords() {
        List<ConsumerRecord<String, Promotion>> list = Arrays.asList(
                new ConsumerRecord<>(
                        topic, 0, 430L, "null",
                        Promotion.newBuilder()
                                .setPromotionId(1234)
                                .setPromotionType("faision")
                                .setDiscount(11)
                                .build()));
        Map<TopicPartition, List<ConsumerRecord<String, Promotion>>> records = new HashMap<>();
        records.put(new TopicPartition(topic, 0), list);
        return new ConsumerRecords<String, Promotion>(records);
    }

    private Properties buildProperties() {
        Properties properties = new Properties();
        properties.setProperty(ConsumerConfig.GROUP_ID_CONFIG, "hi");
        properties.setProperty(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, "http://127.0.0.1:9092");
        properties.setProperty(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringDeserializer.class.getName());
        properties.setProperty(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, KafkaAvroSerialize.class.getName());
        return properties;
    }
}
