package com.safeway.epe.avrokafkawithoutschemaregistry.producer;

import com.safeway.epe.avrokafkawithoutschemaregistry.consumer.PromotionConsumer;
import com.safeway.epe.avrokafkawithoutschemaregistry.serializer.KafkaAvroDeserialize;
import com.safeway.epe.avrokafkawithoutschemaregistry.serializer.KafkaAvroSerialize;
import com.safeway.epe.model.Promotion;
import org.apache.kafka.clients.consumer.KafkaConsumer;
import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.util.Properties;
import java.util.Random;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

@SpringBootTest
public class PromotionProducerIntegrationTest {
    private static final Logger LOGGER = LoggerFactory.getLogger(PromotionProducerIntegrationTest.class);

    @RegisterExtension
    public EmbeddedSingleNodeKafkaCluster kafkaCluster = new EmbeddedSingleNodeKafkaCluster();

    private static final String TOPIC_NAME = "users";
    private KafkaProducer<String, Promotion> producer;
    private KafkaConsumer<String, Promotion> consumer;

    @Autowired
    private PromotionConsumer myConsumer;

    @BeforeEach
    void setUp() throws Exception {
        kafkaCluster.createTopic(TOPIC_NAME);
        producer = kafkaProducer();
        consumer = kafkaConsumer();
        buildAndSendRecord();
    }

    @AfterEach
    void tearDown() {
        producer.close();
        consumer.close();
    }

    @Test
    void testSendPromotionEvent() throws InterruptedException {
        new Thread(() -> myConsumer.consumeMesage(consumer)).start();
        Thread.sleep(10000);
    }

    private KafkaConsumer<String, Promotion> kafkaConsumer() {
        final Properties properties = new Properties();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, kafkaCluster.bootstrapServers());
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, KafkaAvroSerialize.class);
        return new KafkaConsumer<String, Promotion>(properties, new StringDeserializer(), new KafkaAvroDeserialize<>(Promotion.class));
    }


    private KafkaProducer<String, Promotion> kafkaProducer() {
        final Properties properties = new Properties();
        properties.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, kafkaCluster.bootstrapServers());
        properties.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class);
        properties.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, KafkaAvroSerialize.class);
        return new KafkaProducer<>(properties);
    }

    private void buildAndSendRecord() {
        try {
            Promotion promotion = generatedPromotion();
            Future<RecordMetadata> future = producer.send(new ProducerRecord<String, Promotion>(TOPIC_NAME, promotion));
            RecordMetadata metadata = future.get();
            if (metadata.hasOffset() && metadata.hasTimestamp()) {
                LOGGER.info("partition::" + metadata.partition() + "\t Offset::" + metadata.offset());
            } else {
                throw new RuntimeException("Exception ocured");
            }
            producer.close();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (ExecutionException e) {
            e.printStackTrace();
        }
    }

    private static Promotion generatedPromotion() {
        Promotion promotion = null;
        for (int i = 0; i < 10; i++) {
            promotion = Promotion.newBuilder()
                    .setPromotionId(new Random().nextInt(10000000))
                    .setPromotionType("faishon")
                    .setDiscount(new Random().nextInt(20))
                    .build();
        }
        return promotion;

    }
}
